@@ -1,6 +1,7 @@
 import contextlib
 import datetime
 import sys
+from typing import Any, Iterator
 
 import pytest
 from freezegun import freeze_time
@@ -22,11 +23,11 @@ class ModuleWithError:
     __name__ = 'module_with_error'
     __dict__ = {}
 
-    def __init__(self, error_type):
+    def __init__(self, error_type: Any):
         self.error_triggered = False
         self.error_type = error_type
 
-    def __dir__(self):
+    def __dir__(self) -> Any:
         try:
             raise self.error_type()
         finally:
@@ -34,10 +35,10 @@ class ModuleWithError:
 
 
 @contextlib.contextmanager
-def assert_module_with_raised_error(error_type):
+def assert_module_with_raised_error(error_type: Any) -> Iterator[None]:
     """Install a module into sys.modules that raises an error upon invoking
     __dir__."""
-    module = sys.modules['module_with_error'] = ModuleWithError(error_type)
+    module = sys.modules['module_with_error'] = ModuleWithError(error_type)  # type: ignore
 
     try:
         yield
@@ -48,7 +49,7 @@ def assert_module_with_raised_error(error_type):
 
 
 @pytest.mark.parametrize('error_type', [ImportError, TypeError])
-def test_ignore_errors_in_start(error_type):
+def test_ignore_errors_in_start(error_type: Any) -> None:
     with assert_module_with_raised_error(error_type):
         freezer = freeze_time(datetime.datetime(2019, 1, 11, 9, 34))
 
